package Jcg.io.options;


/**
 * An admissible option (description, multiplicity, parameters, ...)
 * By definition an option starts with the '-' character
 * e.g.:
 * 		-f filename.dat
 * 		-t output.txt input.txt
 * 		-r
 */
public class Option {
	int multiplicity;
	String option;
	String[] parameters;
	String definition;
	int type; // String, Integer, Double
	
	/**
	 * Defining an option with a given multiplicity, and a list of parameters
	 */
	public Option(int m, String option, String[] param, String def) {
		this.multiplicity=m;
		this.option=option;
		this.parameters=new String[this.multiplicity];
		this.parameters=param;
		this.definition=def;
		this.type=ProcessInputParameters.TYPE_STRING;
	}
	
	public boolean equals(Object o) {
		Option op=(Option)o;
		return this.option.equals(op);
	}
	
	/**
	 * Defining an option with a given multiplicity, a type and a list of parameters
	 */
	public Option(int m, String option, String[] param, int type) {
		this.multiplicity=m;
		this.option=option;
		this.parameters=new String[this.multiplicity];
		this.parameters=param;
		this.type=type;
	}
	
	/**
	 * Return a String describing the usage of this option
	 */	
	public String toString() {
		String result="\t "+this.option+"\t\t\t";
		result=result+this.definition;
		if(this.parameters==null || this.parameters.length==0)
			return result;
		result=result+"\t { ";
		for(String s: this.parameters)
			result=result+s+" ";
		return result+"}";
	}
	
}

/**
 * For processing input arguments and relative input option
 */
class InputOption {
	String arg;
	String[] param;
	int type;
	
	public InputOption(String arg, String[] param, int type) {
		this.arg=arg;
		this.param=param;
		this.type=type;
	}
	
	public boolean equals(Object o) {
		InputOption op=(InputOption)o;
		return this.arg.equals(op);
	}
	
	/**
	 * Return the parameter value corresponding to a given option
	 */
	public int getIntValue(int i) {
		if(param==null)
			throw new Error("error: parameters not defined");
		if(i<0 || i>=this.param.length)
			throw new Error("error: wrong number of parameters");
		if(this.type!=ProcessInputParameters.TYPE_INTEGER)
			throw new Error("error: wrong type");
		int result=Integer.parseInt(this.param[i]);
		return result;
	}
	
	/**
	 * Return the parameter value corresponding to a given option
	 */
	public double getDoubleValue(int i) {
		if(param==null)
			throw new Error("error: parameters not defined");
		if(i<0 || i>=this.param.length)
			throw new Error("error: wrong number of parameters");
		if(this.type!=ProcessInputParameters.TYPE_DOUBLE)
			throw new Error("error: wrong type");
		double result=Double.parseDouble(this.param[i]);
		return result;
	}

	/**
	 * Return the parameter value corresponding to a given option
	 */
	public String getStringValue(int i) {
		if(param==null)
			throw new Error("error: parameters not defined");
		if(i<0 || i>=this.param.length)
			throw new Error("error: wrong number of parameters");
		if(this.type!=ProcessInputParameters.TYPE_STRING)
			throw new Error("error: wrong type");
		return this.param[i];
	}

}
