package Jcg.io.options;

import java.util.LinkedList;;

/**
 * @author Luca Castelli Aleardi (Ecole Polytechnique, INF562, 2013)
 *
 * A class for processing input options and parameters (input of the main function)
 */
public class ProcessInputParameters {

	// constants defining error messages
	final String helpMessage="use option -help for more details";
	final String missingArgumentsMessage="Missing arguments: waiting for more arguments";
	final String wrongTypeMessage="wrong type argument";
	final String notDefinedOptionMessage="Unrecognized option: ";
	final String usageMessage;
	
	// constants defining the multiplicity of an option
	final static int MULT_ZERO=0;
	final static int MULT_ONE=1;
	final static int MULT_TWO=2;
	
	// constants defining the type of the argument of an option
	public final static int TYPE_STRING=0;
	public final static int TYPE_INTEGER=1;
	public final static int TYPE_DOUBLE=2;
	
	final int minNumberOfArguments; // minimum number of arguments required by the program
	final public LinkedList<Option> admisOptions=new LinkedList<Option>(); // list of admissible options
	
	public LinkedList<String> data=new LinkedList<String>(); // input data
	public LinkedList<InputOption> selectedOptions=new LinkedList<InputOption>(); // input options
	
	public ProcessInputParameters(Option[] options, int min, String[] args, String usageMsg) {
		for(Option op: options)
			this.admisOptions.add(op);
		this.minNumberOfArguments=min;
		this.usageMessage=usageMsg;
		this.processArguments(args);
	}
	
	private int processData(String[] args, int k) {
		if(k>=args.length)
			return -1; // end of the input reached
		this.data.add(args[k]);
		return k+1;
	}
	
	private int processOption(String[] args, int k) {
		if(k<0 || k>=args.length)
			throw new Error("error: wrong input parameters");
		if(args[k].charAt(0)!='-') // the argument is a data, not an option
			return processData(args, k);
		Option o=this.findAdmissibleOption(args[k]);
		if(o==null) {
			System.out.println(this.helpMessage+"\n"+this.notDefinedOptionMessage+args[k]);
			System.exit(0);
			//throw new Error(this.notDefinedOptionMessage);
		}
		if(o.multiplicity==0) {
			this.selectedOptions.add(new InputOption(o.option, null, o.type));
			return k+1;
		}
		else if(o.multiplicity==1) {
			if(k+1>=args.length || args[k+1].charAt(0)=='-') {
				System.out.println(this.helpMessage+"\n (one parameter required)");
				throw new Error(this.missingArgumentsMessage);
			}
			this.selectedOptions.add(new InputOption(o.option, new String[]{args[k+1]}, o.type));
			return k+2;
		}
		else if(o.multiplicity==2) {
			if(k+2>=args.length || args[k+1].charAt(0)=='-' || args[k+2].charAt(0)=='-') {
				System.out.println(this.helpMessage+"\n (two parameters required)");
				throw new Error(this.missingArgumentsMessage);				
			}
			this.selectedOptions.add(new InputOption(o.option, new String[]{args[k+1], args[k+2]}, o.type));
			return k+3;
		}
		else {
			throw new Error("error: multiplicity more than 2 (not allowed)");
		}

	}
	
	private Option findAdmissibleOption(String s) {
		for(Option op: this.admisOptions)
			if(s.equals(op.option)==true)
				return op;
		return null;
	}

	/*private InputOption findSelectedOption(String s) {
		for(InputOption op: this.selectedOptions)
			if(s.equals(op.arg)==true)
				return op;
		return null;
	}*/

	/**
	 * Read and process the input string (arguments of the main function)
	 */
	private void processArguments(String[] args) {
		if(args==null || args.length==0) {
			System.out.println(this.missingArgumentsMessage);
			System.out.println(this.helpMessage);
			System.exit(0);
		}
		if(args.length>0 && args[0].equals("-help")) {
			System.out.println(this);
			System.exit(0);
		}
		if(args.length<this.minNumberOfArguments) {
			System.out.println(this.missingArgumentsMessage);
			System.out.println(this.helpMessage);
			System.exit(0);
		}
		int k=0;
		while(k!=-1 && k<args.length)
			k=this.processOption(args, k);
	}
	
	/**
	 * Return input data (the i-th item)
	 */
	public String getData(int i) {
		if(this.data.isEmpty()) {
			System.out.println("No input data\n"+this.helpMessage);
			throw new Error("Enter input data");
		}
		return this.data.get(i);
	}

	/**
	 * Return input data
	 */
	public String getData() {
		return getData(0);
	}

	/**
	 * Check whether a given option has been selected
	 */
	public boolean isSelected(String s) {
		if(this.selectedOptions.isEmpty()) {
			return false;
		}
		for(InputOption op: this.selectedOptions)
			if(op.arg.equals(s))
				return true;
		return false;
	}

	/**
	 * Return a String describing the usage and all input options for this program
	 */
	public String toString() {
		String result="Usage:\n \t"+this.usageMessage+"\n";
		result=result+"where options include:\n";
		for(Option o: this.admisOptions)
			result=result+o+"\n";
		return result;
	}

}
