package Jcg.schnyderwoods;

import java.awt.Color;
import java.util.ArrayList;

import Jcg.polyhedron.Halfedge;
import Jcg.polyhedron.Polyhedron_3;
import Jcg.geometry.*;

/**
 * @author Luca Castelli Aleardi
 * 
 * Provides methods for representing and manipulating
 * an edge orientation of a map (also known as alpha-orientation)
 * Half-edges are assumed to be numbered, from 0.. (2e-1)
 */
public abstract class EdgeOrientation {

    public Polyhedron_3<Point_> polyhedron; // the input mesh (combinatorial map)
    public boolean[] isWellOriented; // says whether an half-edge is "oriented" toward his target vertex
    public byte[] edgeColor; // stores an integer representing the coloring of an edge

    public boolean[] getEdgeOrientation() {
    	return this.isWellOriented;
    }
    
    public byte[] getEdgeColoration() {
    	return this.edgeColor;
    }
    
	/**
	 * Return a string representing the edge coloring/orientation
	 */
    public String orientationToString() {
    	String result="";
    	
    	for(Halfedge<Point_> e: (ArrayList<Halfedge<Point_>>)this.polyhedron.halfedges) {
    		result=result+e.getOpposite().index+","+e.getVertex().index+", color"+this.edgeColor[e.index]+"\n";
    	}
    	return result;
    }
    
    public Color[] getEdgeColors() {
    	Color[] result=new Color[this.edgeColor.length];
    	
    	for(int i=0;i<this.edgeColor.length;i++) {
    		if(edgeColor[i]==0) result[i]=Color.red;
    		else if(edgeColor[i]==1) result[i]=Color.blue;
    		else if(edgeColor[i]==2) result[i]=Color.black;
    		else if(edgeColor[i]==3) result[i]=Color.orange;
    		else if(edgeColor[i]==5) result[i]=Color.lightGray;
    		
    		else result[i]=Color.gray;
    	}
    	return result;
    }

}
